/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 */

#include <string.h>
#include <plugin.h>
#include <version.h>
#include <savedstatuses.h>
#include <dbus-maybe.h>
#include <dbus-server.h>
#include <dbus-bindings.h>
#include <gio/gio.h>

#define N_(x) x
#define DISPLAY_VERSION "0.2"

static void message(PurplePlugin* plugin,const char* msg)
{
    purple_notify_message(plugin,PURPLE_NOTIFY_MSG_INFO,"Gnome-Shell Connector",msg,NULL,NULL,NULL);
}

#define GNOME_AVAILABLE 0
#define GNOME_INVISIBLE 1
#define GNOME_BUSY 2
#define GNOME_IDLE 3
#define GNOME_NSTATUS 4

static int Purple2Gnome(PurpleStatusPrimitive purple_status)
{
    switch (purple_status)
    {
    case PURPLE_STATUS_OFFLINE:
    case PURPLE_STATUS_INVISIBLE:
        return GNOME_INVISIBLE;
    case PURPLE_STATUS_UNAVAILABLE:
    case PURPLE_STATUS_AWAY:
    case PURPLE_STATUS_EXTENDED_AWAY:
        return GNOME_BUSY;
    case PURPLE_STATUS_AVAILABLE:
        return GNOME_AVAILABLE;
    default:
        return GNOME_INVISIBLE;
    }
}

static PurpleStatusPrimitive Gnome2Purple(int gnome_status)
{
    switch (gnome_status)
    {
    case GNOME_AVAILABLE:
        return PURPLE_STATUS_AVAILABLE;
    case GNOME_INVISIBLE:
        return PURPLE_STATUS_INVISIBLE;
    case GNOME_BUSY:
        return PURPLE_STATUS_AWAY;
    case GNOME_IDLE:
        return PURPLE_STATUS_INVISIBLE;
    }
    return PURPLE_STATUS_INVISIBLE;
}

#define TITLESIZE 512

struct SPluginParams
{
    PurplePlugin* plugin;
    GDBusConnection* connection;
    char status[GNOME_NSTATUS][TITLESIZE];
};
typedef struct SPluginParams PluginParams;


static void InitPluginParams(PluginParams* params,PurplePlugin* plugin,GDBusConnection* connection)
{
    size_t i;
    params->plugin = plugin;
    params->connection = connection;
    for (i = 0; i < GNOME_NSTATUS; ++i)
        params->status[i][0] = '\0';
}

static void SetInGnome(PluginParams* params,int status)
{
    GVariant *ret;
    GError* error = NULL;
    GValue v = {0,{{0}}};
    GDBusProxy* presence = g_dbus_proxy_new_sync(
        params->connection,
        G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
        G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
        NULL,
        "org.gnome.SessionManager",
        "/org/gnome/SessionManager/Presence",
        "org.gnome.SessionManager.Presence",
        NULL, NULL);
    g_value_init(&v,G_TYPE_UINT);
    g_value_set_uint(&v,status);
    ret = g_dbus_proxy_call_sync(presence, "SetStatus", g_variant_new ("(u)", status), G_DBUS_CALL_FLAGS_NONE, -1, NULL, &error);
    g_object_unref (presence);

    if (ret == NULL)
    {
	g_warning ("Failed to set presence: %s", error->message);
        g_error_free(error);
        return;
    }
    g_variant_unref (ret);
}

static void status_signal_handler(GDBusProxy* presence, gchar *sender_name, gchar *signal_name, PluginParams* params, guint gnome_status)
{
    if (g_str_equal (signal_name, "StatusChanged")) {
        PurpleSavedStatus* current_status = purple_savedstatus_get_current();
        guint current_pidgin_status = Purple2Gnome(purple_savedstatus_get_type(current_status));
        if (current_pidgin_status < GNOME_NSTATUS && !purple_savedstatus_is_transient(current_status))
        {
            const char* title = purple_savedstatus_get_title(current_status);
            if (title)
                strcpy(params->status[current_pidgin_status],title);
        }
        if (gnome_status >= GNOME_NSTATUS)
            return;
        if (current_pidgin_status != gnome_status)
        {
            PurpleSavedStatus* new_status = NULL;
            if (params->status[gnome_status][0])
                new_status = purple_savedstatus_find(params->status[gnome_status]);
            if (new_status)
                purple_savedstatus_activate(new_status);
            else
            {
                PurpleSavedStatus* newStatus = purple_savedstatus_new(NULL,Gnome2Purple(gnome_status));
                purple_savedstatus_activate(newStatus);
            }
        }
    }
}

static gboolean plugin_load(PurplePlugin *plugin)
{
    GDBusProxy* presence;
    PurpleSavedStatus* currentStatus;
    GError* error = NULL;
    PluginParams* params = malloc(sizeof(PluginParams));
    plugin->extra = params;
    GDBusConnection* connection = g_bus_get_sync(G_BUS_TYPE_SESSION, NULL, &error);
    if (!connection)
    {
        message(plugin,error->message);
        g_error_free(error);
        return FALSE;
    }
    InitPluginParams(params,plugin,connection);
    presence = g_dbus_proxy_new_sync (
        connection,
        G_DBUS_PROXY_FLAGS_NONE,
        NULL, /* interface info */
        "org.gnome.SessionManager",
        "/org/gnome/SessionManager/Presence",
        "org.gnome.SessionManager.Presence",
        NULL, /* cancellable */
        NULL /* error */);
    currentStatus = purple_savedstatus_get_current();
    SetInGnome(params,Purple2Gnome(purple_savedstatus_get_type(currentStatus)));
    g_signal_connect (presence, "g-signal", G_CALLBACK (status_signal_handler), params);
    g_object_unref (presence);
    return TRUE;
}

static gboolean plugin_unload(PurplePlugin *plugin)
{
    free(plugin->extra);
    return TRUE;
}

static PurplePluginInfo info =
{
    PURPLE_PLUGIN_MAGIC,
    PURPLE_MAJOR_VERSION,
    PURPLE_MINOR_VERSION,
    PURPLE_PLUGIN_STANDARD,
    NULL,                                             /**< ui_requirement */
    0,                                                /**< flags          */
    NULL,                                             /**< dependencies   */
    PURPLE_PRIORITY_DEFAULT,                          /**< priority       */
    "gtk-2925578-gnomeshell",                         /**< id             */
    N_("Gnome Shell Connector"),                      /**< name           */
    DISPLAY_VERSION,                                  /**< version        */
    N_("Connects Pidgin to Gnome Shell Status"),      /**  summary        */
    N_("Gnome Shell Connector"),                      /**  description    */
    "Wojtek Surówka",                                 /**< author         */
    "http://gnomeshpidgin.sourceforge.net",           /**< homepage       */
    plugin_load,                                      /**< load           */
    plugin_unload,                                    /**< unload         */
    NULL,                                             /**< destroy        */
    NULL,                                             /**< ui_info        */
    NULL,                                             /**< extra_info     */
    NULL,                                             /**< prefs_info     */
    NULL,
    /* padding */
    NULL,
    NULL,
    NULL,
    NULL
};

static void init_plugin(PurplePlugin *plugin)
{
}

PURPLE_INIT_PLUGIN(dbus_example, init_plugin, info)

